@@ -18,15 +18,20 @@ from mundipy.api.osm import grab_from_osm
 from mundipy.pcs import choose_pcs
 
 class MundiQ:
-    def __init__(self, center, mapdata, plot_target=None, pcs=None, clip_distance=500):
+    def __init__(self, center, mapdata, plot_target=None, units='meters', clip_distance=500):
+        self.pcs = choose_pcs(box(*center.geometry.bounds), units=units)['crs']
+
         # a row in a GeoDataFrame, with a column called .geometry
         # in local projected coordinate system
         self.center = center
+
+        reproject = pyproj.Transformer.from_crs(pyproj.CRS('EPSG:4326'),
+            pyproj.CRS(self.pcs), always_xy=True).transform
+        self.center.geometry = transform(reproject, self.center.geometry)
+
         # GeoPool
         self.mapdata = mapdata
 
-        self.pcs = pcs
-
         # could be matplotlib axes or 'geojson'
         self.plot_target = plot_target
         self.plot_legend = dict()
@@ -115,7 +120,9 @@ class MundiQ:
             self.plot_handles.append(mpatches.Patch(color=color, label=name))
 
         if self.plot_target == 'geojson':
-            self.plot_contents.append((shape, mcolors.CSS4_COLORS[color]))
+            # plot_contents can only contain shapely geometries
+            for idx, row in shape.items():
+                self.plot_contents.append((row, mcolors.CSS4_COLORS[color]))
         else:
             shape.plot(ax=self.plot_target, edgecolor='black', facecolor=color, aspect='equal', alpha=0.75)
 
@@ -127,8 +134,7 @@ class Mundi:
 
         if units not in ['meters', 'feet']:
             raise TypeError('units passed to Mundi() was neither meters nor feet')
-
-        self.pcs = choose_pcs(box(*self.main.dataframe.geometry.total_bounds), units=units)['crs']
+        self.units = units
 
     def plot(self, fn, clip_distance=500, output_type='matplotlib'):
         if output_type == 'geojson':
@@ -136,26 +142,24 @@ class Mundi:
         elif output_type == 'matplotlib':
             fig, ax = plt.subplots()
 
-        df = self.main.dataframe.drop_duplicates(subset=['geometry']).to_crs(crs=self.pcs)
-
-        Q = MundiQ(df.iloc[1], self.mapdata, plot_target=('geojson' if output_type == 'geojson' else ax), pcs=self.pcs, clip_distance=clip_distance)
+        # TODO: drop duplicates, except it's very slow
+        #.drop_duplicates(subset=['geometry'])
+        Q = MundiQ(self.main.dataframe.iloc[1], self.mapdata, plot_target=('geojson' if output_type == 'geojson' else ax), units=self.units, clip_distance=clip_distance)
         res = fn(Q)
 
         if output_type == 'geojson':
             # convert to dfs
-            dfs = list(map(lambda shp: gpd.GeoDataFrame(data={ 'geometry': shp[0] }, crs=self.pcs).to_crs(epsg=4326), Q.plot_contents))
-            for i, df in enumerate(dfs):
-                df['fill'] = Q.plot_contents[i][1]
+            dfs = gpd.GeoDataFrame(data=Q.plot_contents, crs='EPSG:4326', columns=['geometry', 'fill'], geometry='geometry')
 
             # 'http://geojson.io/#data=data:application/json,%s' % urllib.parse.quote(
-            return gpd.GeoDataFrame(data=pd.concat(dfs)).to_json()
+            return dfs.to_json()
         elif output_type == 'matplotlib':
             ax.legend(handles=Q.plot_handles, loc='upper right')
             plt.show()
 
     def q(self, fn, progressbar=False):
         # make iterator unique by geometry
-        unique_iterator = self.main.dataframe.drop_duplicates(subset=['geometry']).to_crs(crs=self.pcs)
+        unique_iterator = self.main.dataframe.drop_duplicates(subset=['geometry'])
 
         res_keys = None
         res_outs = dict()
@@ -165,7 +169,7 @@ class Mundi:
             finiter = tqdm(finiter, total=len(unique_iterator))
 
         for window in finiter:
-            Q = MundiQ(window, self.mapdata, pcs=self.pcs)
+            Q = MundiQ(window, self.mapdata)
             res = fn(Q)
 
             # coerce to tuple
@@ -185,4 +189,4 @@ class Mundi:
                 res_outs[key].append(val)
             res_outs['geometry'].append(window.geometry)
 
-        return gpd.GeoDataFrame(res_outs, crs=self.pcs, geometry='geometry').to_crs(crs='EPSG:4326')
+        return gpd.GeoDataFrame(res_outs, crs='EPSG:4326', geometry='geometry').to_crs(crs='EPSG:4326')
